import logging

import boto3
import salt.utils.json

log = logging.getLogger(__name__)

CLIENT = None


def publish(opts, topic, data, qos=1):
    global CLIENT

    if CLIENT is None:
        CLIENT = boto3.client(
            "iot-data",
            region_name=opts.get("aws_region"),
            endpoint_url=opts.get("endpoint"),
        )

    CLIENT.publish(
        topic=f"{topic}",
        qos=qos,
        # retain=False, #depends on boto version
        payload=bytes(salt.utils.json.dumps(data), "utf-8"),
    )



"""
Salt returner module
"""
import logging
from typing import Any

import handlers.awsiot as awsiot
import handlers.mqtt as mqtt
import salt.returners

log = logging.getLogger(__name__)

__virtualname__ = "mqtt_return"


def __virtual__():
    return __virtualname__


def _get_options(ret=None):
    defaults = {
        "endpoint": "localhost",
        "port": 1883,
        "output": "mqtt",
        "client_id": "salt-master",
    }

    attrs = {
        "endpoint": "endpoint",
        "port": "port",
        "output": "output",
        "topic_prefix": "",
        "aws_access_key_id": "aws_access_key_id",
        "aws_secret_access_key": "aws_secret_access_key",
        "aws_region": "aws_region",
        "client_id": "client_id",
    }

    _options = salt.returners.get_returner_options(
        f"returner.{__virtualname__}",
        ret,
        attrs,
        __salt__=__salt__,
        __opts__=__opts__,
        defaults=defaults,
    )
    # Ensure port is an int
    if "port" in _options:
        _options["port"] = int(_options["port"])
    return _options


def event_return(events):
    _options = _get_options()

    handler = _get_handler(_options)

    for event in events:
        topic = event.get("tag", "")
        data = event.get("data", "")

        # Add prefix if specified
        if _options.get("topic_prefix"):
            topic = f"{_options.get('topic_prefix')}/{topic}"

        try:
            handler(
                opts=_options,
                topic=topic,
                data=data,
            )
        except Exception as error:
            log.error(data)
            log.error(error)


def _get_handler(opts) -> Any:
    if opts.get("output") == "mqtt":
        return mqtt.publish
    elif opts.get("output") == "awsiot":
        return awsiot.publish



import pytest
import saltext.mqtt_return.returners.mqtt_return_mod as mqtt_return_returner


@pytest.fixture
def configure_loader_modules():
    module_globals = {
        "__salt__": {"this_does_not_exist.please_replace_it": lambda: True},
    }
    return {
        mqtt_return_returner: module_globals,
    }


def test_replace_this_this_with_something_meaningful():
    assert "this_does_not_exist.please_replace_it" in mqtt_return_returner.__salt__
    assert mqtt_return_returner.__salt__["this_does_not_exist.please_replace_it"]() is True



import paho.mqtt.publish as mqtt_pub
import salt.utils.json


def publish(opts, topic, data, qos=1):
    mqtt_pub.single(
        f"{topic}",
        payload=bytes(salt.utils.json.dumps(data), "utf-8"),
        qos=qos,
        hostname=opts.get("endpoint", ""),
        port=opts.get("port", ""),
        client_id=opts.get("client_id"),
    )
